/*
информация о каналах реле содержится в EEPROB по два байта на каждую минуту как представлено ниже 
                  Байт 1                  Байт 2
           7  6  5  4  3  2  1 0      7 6 5 4 3 2 1 0
каналы    16 15 14 13 12 11 10 9      8 7 6 5 4 3 2 1 

на каждую минуту свое состояние реле 0-выключено 1-включено

Пямять EEPROM
 0    1         0 мин
 2    3         1 мин
 4    5         2 мин
...            ....
2878 2879     1439 мини = 23часа 59мин

2881      минимальная температура. отключения охлаждения
2882      максимальная температура. включения охлаждения
  
2885      номер варианта LED подсветки если 0 то автоматически поочереди

2886      состояние Service


2887      по каким дням недели осуществлять подмену воды            Байт                
                                                             7   6   5   4   3   2   1   0 
                                                             0  Пн. Вт. Ср. Чт. Пт. Сб. Вс.    
время включения слива                                                             
2888  2889  понедельник час мин
2890  2891  вторник час мин                                                           
2892  2893  среда час мин
2894  2895  четверг час мин
2896  2897  пятница час мин                                                           
2898  2899  суббота час мин
2900  2901  воскресения час мин

количество сливаемых литров
2902  понедельник  
2903  вторник
2904  среда
2905  четверг
2906  пятница
2907  суббота
2908  воскресения

2909 мл.байт временной задержки LED
2910 ст.байт временной задержки LED
2911 LED_MIN
2912 LED_MAX
2913 К   коэфициент смещения лампочек
2914

*/


/* Подключаем библиотеку для работы с LCD */
#include <LiquidCrystal.h>
#include <Wire.h>
#include <OneWire.h>
#include <DallasTemperature.h>
#include <DS1307RTC.h>
#include <Time.h>
#include <TimerOne.h>
#include <EEPROM.h>

#include <SPI.h>
#include <SD.h>
#include <Ethernet.h>
#include <EthernetUdp.h>     

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//Пин подключения клавиатуры
#define ButtonPin     A0    

//Пин подключения термодатчика
#define TemperPin     22     //30
#define TemperPin_2   A9   //18   Открыть красный

//Пин подключения датчика потока воды
#define WaterPin      19     //31   

//Пин подключения ethernet shild
#define SD_SS         4 
#define Ethernet_SS   10 

//Пины подключения ночной подсветки
#define LEDPin_1  44         //23   Куда подключен 1 светодиод подсветки
#define LEDPin_2  46         //24   Куда подключен 2 светодиод подсветки
#define LEDPin_3  3          //25   Куда подключен 3 светодиод подсветки
#define LEDPin_4  2         //26   Куда подключен 4 светодиод подсветки

//Пины подключения клапанов и индикаторов открытия клапанов
#define SensorPin_1  24      //27   Куда подключен 1 датчик уровня воды    
#define SensorPin_2  26      //28   Куда подключен 2-й датчик уровня воды
#define SensorPin_A  28      //29   Куда подключен аварийный датчик уровня воды

//Выводы реле
#define RelePin_1    23      //3   Свет1       (220)
#define RelePin_2    25      //4   Свет2       (220)
#define RelePin_3    27      //5   Свет3       (220)
#define RelePin_4    29      //6   Свет4       (220)
#define RelePin_5    31      //7   Нагреватель (220)
#define RelePin_6    33      //8   Фильтр      (220)
#define RelePin_7    35      //9   Воздух      (220)
#define RelePin_8    37      //10  Нагрев грунт(220)
#define RelePin_9    39      //11  Удобрение канал 1         (12)
#define RelePin_10   41      //12  Удобрение канал 2         (12)
#define RelePin_11   43      //13  Магнитный клапан вход СО2 (12) 
#define RelePin_12   47      //14  Вулкан         (12)
#define RelePin_13   42      //15  Клапан вода вход 2      (12)
#define RelePin_14   49      //16  Клапан вода вход 1      (12)
#define RelePin_15   48      //17  Клапан вода вход общий  (12)

//Управление клапана слива
#define KlapanPin   A8   //19   Управление клапаном слива

//Транзисторное управление
#define Cooling_Pin     40   //20   Вентиляторы охлаждения
#define Ventilation_Pin 38   //21   Вентиляторы вентиляции
#define Feeder_Pin      36   //22   Кормушка

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

/* Создаём объект LCD-дисплея, используя конструктор класса LiquidCrystal
* с 6ю аргументами. Библиотека по количеству аргументов сама определит,
* что нужно использовать 4-битный интерфейс.
* Указываем, к каким пинам Arduino подключены выводы дисплея:
*   RS   8
*   E    9
*   DB4  4
*   DB5  5
*   DB6  6
*   DB7  7
*/
LiquidCrystal lcd(8, 9, 14, 5, 6, 7);

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//Определение переменной для работы с RCT
tmElements_t tm;                    

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//Определение параметров датчика температуры
OneWire oneWire(TemperPin);         // Определяем протокол для работы с температурным датчиком
DallasTemperature sensors(&oneWire);// Определяем протокол для работы с температурным датчиком

OneWire oneWire_2(TemperPin_2);         // Определяем протокол для работы с температурным датчиком
DallasTemperature sensors_2(&oneWire_2);// Определяем протокол для работы с температурным датчиком

DeviceAddress tempDeviceAddress, tempDeviceAddress_2;//, insideThermometer, outsideThermometer;    // Определяем адресс температурного датчика

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//Определение параметров для работы по сети
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
IPAddress ip(192, 168, 1, 117);
unsigned int localPort = 1986;      // локальный порт

// buffers for receiving and sending data
char packetBuffer[UDP_TX_PACKET_MAX_SIZE]; //буфер приема пакета по UDP

IPAddress remote;  //Переменая для хранения адреса и порта отправившего сообщение

//EthernetUDP 
EthernetUDP Udp;

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

byte WDAY=1;  //Переменная для хранения дней недели 1-понедельник 2-втрник ....7-воскресение

byte OldMin=0;  //Переменная для хранения предыдущего времени

byte CountMenu = 0;  //Счетчик позиции меню

int SlivMin = 0;   //Счетчик минут времени слива
int SlivDlit = 0;  //Время слива

byte Displey = 2; //Вариант отображения дисплея

byte LED = 5;       //Режим ночной подсветки
byte LED_MIN = 0;   //Минимальный уровень яркости
byte LED_MAX = 150; //Максимальный уровень яркости
int TimeLED = 50;   //Время задержки увеличения на единицу величины ШИМ на светодиодах в милисек
byte K = 30;        //Коэфициэт смещения при нахлесте лампочек

byte TemperaturaMax;              //Переменная для хранения температуры включающей охлаждения
byte TemperaturaMin;              //Переменная для хранения температуры отключиющей охлаждения
float TemperaturaSred;            //Переменная для хранения средней температуры 
float Temperatura_Water = 0.00;  //Температура воды считанная с датчика 
float Temperatura_Ground = 0.00; //Температура грунта считанная с датчика 
unsigned long Old_Time_DS1820 = 0;//Расчет времени для считывания температуры

int Time_Hallow_Prog = 0;  //Счетчик отсутствия приветствий 
char BuffSendUDP[256];     //Буфер для сообщений отправляемых по UDP
String Tabl;

boolean selup = false, seldown = true, lef = false, dow = false, dowdown = false, up = false, updown = false, rigt = false;  //Флаги нажатия и отжатия кнопок клавиатуры
boolean TemperVkl = false;  //флаг включения вытежки по температуре чтобы не выключить по времени во время охлаждения
boolean Korm_ON = false;       //флаг начала кормления
boolean SLIV = false;       //флаг начала слива воды
boolean SLIV_NAGREV = true; //флаг отключения нагревателя при сливе
boolean ServicesSTOP = true;//Флаг включения сервисной функции когда все каналы отключены 
boolean LEDON = false;      //Флаг включения ночной подсветки
boolean Start_LED1 = true;   //флаг первого включения ночной подсветки
boolean Start_LED2 = true;   //флаг первого включения ночной подсветки
boolean Start_LED3 = true;   //флаг первого включения ночной подсветки
boolean Start_LED4 = true;   //флаг первого включения ночной подсветки
boolean Start_LED5 = true;   //флаг первого включения ночной подсветки
boolean Kad = false;         //Флаг указывающи на начало кадра приходящего через порт
boolean SD_OK = false;       //Флаг наличия SD карты
boolean HellowProg = false;  //Флаг связи с програппой (если true то программа шлет запросы раз в сек) 
boolean Temperatura_Water_ERROR = false;  //Флаг неисправности датчика температуры воды
boolean Temperatura_Ground_ERROR = false; //Флаг неисправности датчика температуры грунта 
//boolean ReadUDP_OK = true; //Флаг разрешения чтения UDP пакетов

String TEMP = "";  // 

byte brightness_1 = 1;  //началиная яркость
int fadeAmount_1 = 1;  //шаг приращения
byte brightness_2 = 1;  //началиная яркость
int fadeAmount_2 = 1;  //шаг приращения
byte brightness_3 = 1;  //началиная яркость
int fadeAmount_3 = 1;  //шаг приращения
byte brightness_4 = 1;  //началиная яркость
int fadeAmount_4 = 1;  //шаг приращения

volatile uint32_t PulsesSensor = 0;     //Счетчик импульсов с датчика
volatile uint8_t  LastFlowPinStart;     //
volatile uint32_t LastFlowRateTime = 0; //
volatile float FlowRate;                //Частота импульсов с датчика
float CountWaterLiter;                  //Количество пройденных литров

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//Функция прерывания по импульсу с датчика расхода воды
SIGNAL(TIMER0_COMPA_vect)
{
  uint8_t x = digitalRead(WaterPin);

  if(x == LastFlowPinStart)
  {
    LastFlowRateTime++;
    return;
  }

  if(x == HIGH)
  {
    PulsesSensor++;
    if(PulsesSensor >= 4000000000) PulsesSensor = 0; 
  }
  
  LastFlowPinStart = x;
  FlowRate = 1000.0;
  FlowRate /= LastFlowRateTime;
  LastFlowRateTime = 0; 
}

//==================================================================================================================================================

//Функция прерывания по импульсу с датчика расхода воды
void useInterrupt(boolean v)
{
  if(v)
  {
    OCR0A = 0xAF;
    TIMSK0 |= _BV(OCIE0A);
  }
  else
  {
    TIMSK0 &= ~_BV(OCIE0A);
  }
}

//==================================================================================================================================================

//Функция настройки программы
void setup()
{
 /* Инициализируем дисплей: 2 строки по 16 символов */
  lcd.begin(16, 2);

//+++++++++++++++++++++++++++++

//Выводим приветствие 
  lcd.setCursor(2, 0);    //Позиция на экране первая строка с третьего символа счет идет с 0
  lcd.print("ARDUINO-AQUA");
  lcd.setCursor(6, 1);    //Позиция на экране первая строка с третьего символа счет идет с 0
  lcd.print("V3.2");
  delay(3000); 

//+++++++++++++++++++++++++++++

 /* Инициализируем COM порт */
//  Serial.begin(1200);
  
  sensors.begin();  //Инициализируем температурный датчик
  sensors_2.begin();  //Инициализируем температурный датчик

//  sensors.getAddress(insideThermometer, 0);
//  sensors.getAddress(outsideThermometer, 1); 
  
//  sensors.setResolution(insideThermometer, 9);
//  sensors.setResolution(outsideThermometer, 9);

//+++++++++++++++++++++++++++++

  pinMode( Ethernet_SS, OUTPUT); //Порт для SS
  pinMode( SD_SS, OUTPUT); //Порт для SC

  Ethernet.begin(mac, ip);
  if(SD.begin(4)) SD_OK = true;
  else            SD_OK = false;   

  Udp.begin(localPort);
//+++++++++++++++++++++++++++++

  pinMode(ButtonPin, INPUT);  //Порт для клавиатуры как вход

//Выходы управления реле
  pinMode(RelePin_1, OUTPUT); //Порт для реле как выход
  pinMode(RelePin_2, OUTPUT); //Порт для реле как выход
  pinMode(RelePin_3, OUTPUT); //Порт для реле как выход
  pinMode(RelePin_4, OUTPUT); //Порт для реле как выход
  pinMode(RelePin_5, OUTPUT); //Порт для реле как выход
  pinMode(RelePin_6, OUTPUT); //Порт для реле как выход
  pinMode(RelePin_7, OUTPUT); //Порт для реле как выход
  pinMode(RelePin_8, OUTPUT); //Порт для реле как выход
  pinMode(RelePin_9, OUTPUT); //Порт для реле как выход
  pinMode(RelePin_10, OUTPUT); //Порт для реле как выход
  pinMode(RelePin_11, OUTPUT); //Порт для реле как выход
  pinMode(RelePin_12, OUTPUT); //Порт для реле как выход
  pinMode(RelePin_13, OUTPUT); //Порт для реле как выход
  pinMode(RelePin_14, OUTPUT); //Порт для реле как выход
  pinMode(RelePin_15, OUTPUT); //Порт для реле как выход

  digitalWrite( RelePin_1, HIGH);   //Высокий уровень на реле размыкает их, канал выключен
  digitalWrite( RelePin_2, HIGH);   //Высокий уровень на реле размыкает их, канал выключен  
  digitalWrite( RelePin_3, HIGH);   //Высокий уровень на реле размыкает их, канал выключен  
  digitalWrite( RelePin_4, HIGH);   //Высокий уровень на реле размыкает их, канал выключен  
  digitalWrite( RelePin_5, HIGH);   //Высокий уровень на реле размыкает их, канал выключен  
  digitalWrite( RelePin_6, HIGH);   //Высокий уровень на реле размыкает их, канал выключен  
  digitalWrite( RelePin_7, HIGH);   //Высокий уровень на реле размыкает их, канал выключен  
  digitalWrite( RelePin_8, HIGH);   //Высокий уровень на реле размыкает их, канал выключен  
  digitalWrite( RelePin_9, HIGH);   //Высокий уровень на реле размыкает их, канал выключен  
  digitalWrite( RelePin_10, HIGH);  //Высокий уровень на реле размыкает их, канал выключен  
  digitalWrite( RelePin_11, HIGH);  //Высокий уровень на реле размыкает их, канал выключен  
  digitalWrite( RelePin_12, HIGH);  //Высокий уровень на реле размыкает их, канал выключен  
  digitalWrite( RelePin_13, HIGH);  //Высокий уровень на реле размыкает их, канал выключен  
  digitalWrite( RelePin_14, HIGH);  //Высокий уровень на реле размыкает их, канал выключен  
  digitalWrite( RelePin_15, HIGH);  //Высокий уровень на реле размыкает их, канал выключен  
  
  pinMode(LEDPin_1, OUTPUT); //Порт для диода подсветки как выход 
  pinMode(LEDPin_2, OUTPUT); //Порт для диода подсветки как выход
  pinMode(LEDPin_3, OUTPUT); //Порт для диода подсветки как выход
  pinMode(LEDPin_4, OUTPUT); //Порт для диода подсветки как выход

  pinMode(KlapanPin, OUTPUT);   //Порт для сигнала открытия клапана слива
  digitalWrite(KlapanPin, LOW); 
  
  pinMode(SensorPin_1, INPUT);   //Порт для входа датчика уровня как вход
  pinMode(SensorPin_2, INPUT);   //Порт для входа датчика уровня как вход
  pinMode(SensorPin_A, INPUT);   //Порт для входа датчика уровня как вход

  pinMode(Cooling_Pin, OUTPUT);     //Порт для вентиляторов охлаждения
  pinMode(Ventilation_Pin, OUTPUT); //Порт для вентиляторов вентиляции
  pinMode(Feeder_Pin, OUTPUT);      //Порт для кормушки

//Устанавливаем прерывание по возрастающему фронту с датчика потока
  pinMode(WaterPin, INPUT);  //Порт для датчика потока как вход
  digitalWrite( WaterPin, HIGH); 
  LastFlowPinStart = digitalRead(WaterPin);
  useInterrupt(true);

  TemperaturaMin = EEPROM.read(2881);  //Считываем с EEPROM минимальную температуру отключения охлаждения
  TemperaturaMax = EEPROM.read(2882);  //Считываем с EEPROM максимальную температуру включения охлаждения
  TemperaturaSred = (TemperaturaMin+TemperaturaMax)/2.0; //Расчет средней температуры

  sensors.requestTemperatures(); 
  Temperatura_Water = sensors.getTempCByIndex(0);   //Читаем температуру воды
  sensors_2.requestTemperatures(); 
  Temperatura_Ground = sensors_2.getTempCByIndex(0);  //Читаем температуру грунта
  
  LED = EEPROM.read(2885);  //вариант подсветки
  TimeLED = EEPROM.read(2909) + (EEPROM.read(2910)<<8); //Считываем с EEPROM задержку для ШИМ светодиодов  
  LED_MIN = EEPROM.read(2911);
  LED_MAX = EEPROM.read(2912);
  K =       EEPROM.read(2913);
  
  if(EEPROM.read(2886) > 0) ServicesSTOP = true;
  else                      ServicesSTOP = false;

  Timer1.initialize(1000000); // Устанавливаем время прерывания по таймеру 1000000 microseconds = 1сек
  Timer1.attachInterrupt( timerIsr ); // прерывание по таймеру вызывает функцию timerIsr  

  lcd.clear();    //Очищаем экран

//+++++++++++++++++++++++++++++


  if(!ServicesSTOP)
  {
    lcd.setCursor(2, 1);
    lcd.print("SERVICES ON");
  }
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
String Tim = "",Dat = ""; //Переменные для хранения даты и времени

//Основной цикл программы
void loop()
{

//=================================================================================================================

//Ночная подсветка
  NightLED(LEDON);

//=================================================================================================================

//Обмен данными по сети
  ReadEthernetUDP();

//=================================================================================================================

//Чтение кнопок
  ReadKey();

//=================================================================================================================

//Вывод времени
 if(CountMenu==0) //Если не включено меню
 {
   Tim = "";
   Dat = ""; //Переменные для хранения даты и времени

   if (RTC.read(tm))  //Чибаем дату и время из RTC 
   {
     //Формируем строку с датой
     Dat += String(print2digits(tm.Day));
     Dat += String('.');
     Dat += String(print2digits(tm.Month));
     Dat += String('.');
     Dat += String(tmYearToCalendar(tm.Year));

     //Формируем строку со временем
     Tim += print2digits(tm.Hour);      //Заносим в строку часы если меньше 10 то добавляем 0 пример 09
     Tim += String(':');
     Tim += print2digits(tm.Minute);    //Заносим в строку минуты если меньше 10 то добавляем 0 пример 09
     Tim += String(':');
     Tim += print2digits(tm.Second);    //Заносим в строку секунды если меньше 10 то добавляем 0 пример 09
     
     WDAY = tm.Wday;   //Считываем день недели
      
    //++++++++++++++++++++++++++++++++++++++++++++++

     //Читаем состояние температурного датчика
     unsigned long Time_DS1820 = millis();
     if((Time_DS1820 - Old_Time_DS1820) >5000 || Old_Time_DS1820 == -100.0) 
     {
       Old_Time_DS1820 = Time_DS1820;
       ReadDS18B20();
     }  

    //++++++++++++++++++++++++++++++++++++++++++++++

     if(OldMin != tm.Minute && ServicesSTOP) // Проверяем если прошла минута и не стоит сервисный запрет то
     {
        Korm_ON = false;

       //++++++++++++++++++++++++++++++++++++++++++++++

       //Если включен слив воды
       if(SLIV) 
       {
         SlivMin++;
         if(SlivMin >= 15) SlivStop();   //Проверяем не вышло ли время слива, если вышло то отключаем слив воды
       }

       //++++++++++++++++++++++++++++++++++++++++++++++

       //Проверка таблицы времен и включение соответствующего канала
       ReadTabsSetRelay();      
       
       //++++++++++++++++++++++++++++++++++++++++++++++

       //Если выбран вариант дисплея 2 то определяем и отображаем следующее событие
       if(Displey == 2) SetMessage((tm.Hour*60)+tm.Minute); 
       
     }//if(OldMin != tm.Minute && ServicesSTOP)  
     
     OldMin = tm.Minute;  //сохраняем текущее количество минут
     
     //================================================

     //Чтение датчиков уровня и управление клапанами
     ReadWaterLevelSensor();  

     //++++++++++++++++++++++++++++++++++++++++++++++
     
     //Проверка не слили ли необходимое количество
     if(SLIV)
     {
       CountWaterLiter = PulsesSensor;
       CountWaterLiter /= 4.8;
       CountWaterLiter /= 60.0;
       if(CountWaterLiter >= SlivDlit) {SlivStop(); SetMessage((tm.Hour*60)+tm.Minute+1);} //если слито необходимое количество литров то останавливаем слив 
     }

     //++++++++++++++++++++++++++++++++++++++++++++++

//Варианты экрана
     //Вариант 1 дата и время
     if(Displey == 1) 
     {
       lcd.setCursor(3, 0);  //Устанавливаем позицию
       lcd.print(Dat);       //Выводи температуру

       lcd.setCursor(4, 1);  //Устанавливаем позицию
       lcd.print(Tim);       //Выводи время
     }
     else
     //Вариант 2 Время температура воды и следующее событие
     if(Displey == 2) 
     {
       lcd.setCursor(0, 0);  //Устанавливаем позицию
       lcd.print(Tim);       //Выводи время

       lcd.setCursor(9, 0);  //Устанавливаем позицию
       if(!Temperatura_Water_ERROR) lcd.print(Temperatura_Water);//Dat);       //Выводи температуру
       else                         lcd.print("ERROR");   

       lcd.setCursor(14, 0);  //Устанавливаем позицию
       lcd.print("C");  //Выводим символ градусов цельсия после температуры
       
       if(SLIV)
       {
         lcd.setCursor(0, 1);  //Устанавливаем позицию
         lcd.print("                ");  //Стираем строку

         lcd.setCursor(3, 1);  //Устанавливаем позицию
         lcd.print(SlivDlit);  //Количество которое нужно слить
         lcd.setCursor(5, 1);  //Устанавливаем позицию
         lcd.print("L = ");    //Выводим символ литров
         lcd.setCursor(9, 1);  //Устанавливаем позицию
         lcd.print(CountWaterLiter);       //Выводим количество слитых литров
         lcd.setCursor(14, 1);  //Устанавливаем позицию
         lcd.print("L");       //Выводим символ литров
       }
     }
     //Вариант 3 Температура воды и Температура грунта
     if(Displey == 3) 
     {
       lcd.setCursor(0, 0);          //Устанавливаем позицию
       lcd.print("Water   ");       //

       lcd.setCursor(8, 0);           //Устанавливаем позицию
       if(!Temperatura_Water_ERROR) lcd.print(Temperatura_Water);//Dat);       //Выводи температуру
       else                         lcd.print("ERROR");   

       lcd.setCursor(13, 0);  //Устанавливаем позицию
       lcd.print("C");        //Выводим символ градусов цельсия после температуры

       lcd.setCursor(0, 1);     //Устанавливаем позицию
       lcd.print("Ground  ");  //

       lcd.setCursor(8, 1);            //Устанавливаем позицию
       if(!Temperatura_Ground_ERROR) lcd.print(Temperatura_Ground);//Dat);       //Выводи температуру
       else                          lcd.print("ERROR");   

       lcd.setCursor(13, 1);  //Устанавливаем позицию
       lcd.print("C");       //Выводим символ градусов цельсия после температуры
     }
     
     if(HellowProg) 
     { 
       lcd.setCursor(15, 0);  //Устанавливаем позицию
       lcd.print("P");//Dat);       //Выводи температуру
     }
     else  
     { 
       lcd.setCursor(15, 0);  //Устанавливаем позицию
       lcd.print(" ");//Dat);       //Выводи температуру
     }


//===================================================================================
   } //if (RTC.read(tm))
   else 
   {
     //Проверка на неисправность часов реального времени
     if (RTC.chipPresent()) 
     {
       lcd.setCursor(0, 0);
       lcd.print("The DS1307 stop");
       lcd.clear();
     } 
     else 
     {
       lcd.setCursor(0, 0);
       lcd.print("The DS1307 error");
       lcd.clear();
     }
     delay(9000);
   }
 }//if(CountMenu==0)
 else
 { 
 }
//==========================================================================
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//Функция добавляет нолик перед односимвольной цифрой к примеру 09
String print2digits(int number) 
{
  String rez;
  if (number >= 0 && number < 10) rez += String('0');
  rez += String(number);
  return rez;
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

